==> tails-perl5lib_0.5.orig.tar.gz: Tails-perl5lib-0.5/lib/Tails/RunningSystem.pm <==
=head1 NAME

Tails::RunningSystem - class that represents the running Tails system

=cut

package Tails::RunningSystem;

use Moose;
use MooseX::Method::Signatures;
use MooseX::Types::Moose qw( :all );
use MooseX::Types::Path::Class;
use MooseX::Has::Sugar::Saccharin;

with 'Tails::Role::HasEncoding';
with 'Tails::Role::DisplayError::Gtk2';

use 5.10.1;
use namespace::autoclean;
use warnings FATAL => 'all';
use autodie qw(:all);

use Carp;
use Carp::Assert::More;
use Data::Dumper;
use Path::Class;
use Sys::Statistics::Linux::MemStats;
use Tails::UDisks;
use Try::Tiny;

use Locale::gettext;
use POSIX;
setlocale(LC_MESSAGES, "");
textdomain("tails-perl5lib");


=head1 ATTRIBUTES

=cut

has 'upgrade_description_url_schema_version' => lazy_build ro Int;

has "$_" => lazy_build ro Str
    for (
        qw{baseurl product_name product_version build_target channel},
        qw{upgrade_description_file_url upgrade_description_sig_url},
        qw{dev_dir os_release_file proc_dir},
    );

has 'udisks' =>
    lazy_build ro 'Tails::UDisks',
    handles => [ qw{device_installed_with_tails_installer get_device_property underlying_physical_device} ];

has 'liveos_mountpoint' => (
    isa        => 'Path::Class::Dir',
    is         => 'rw',
    lazy_build => 1,
    coerce     => 1,
    documentation => q{Mountpoint of the Tails system image.},
);

has 'boot_device' =>
    lazy_build rw Str,
    documentation => q{The UDI of the physical block device where Tails is installed, e.g. /org/freedesktop/UDisks/devices/sdb.};

has 'system_partition' =>
    lazy_build rw Str,
    documentation => q{The UDI of the partition where Tails is installed, e.g. /org/freedesktop/UDisks/devices/sdb1.};

has 'constants' =>
    lazy_build ro 'Tails::Constants',
    metaclass => 'NoGetopt',
    handles   => [ qw{system_partition_label}];

has 'main_window' => ro 'Gtk2::Window';

foreach (qw{boot_device_vendor boot_device_model}) {
    has $_ => lazy_build ro Str;
}

has 'override_started_from_device_installed_with_tails_installer' =>
    lazy_build ro Str;


=head1 CONSTRUCTORS AND BUILDERS

=cut

method _build_upgrade_description_url_schema_version { 1 }
method _build_dev_dir         { '/dev' }
method _build_os_release_file { '/etc/os-release' }
method _build_proc_dir        { '/proc' }
method _build_product_name    { $self->os_release_get('TAILS_PRODUCT_NAME') }
method _build_product_version { $self->os_release_get('TAILS_VERSION_ID')   }
method _build_baseurl         { 'https://tails.boum.org' }
method _build_udisks          { Tails::UDisks->new(); }

method _build_build_target {
    my $arch = `dpkg --print-architecture`; chomp $arch; return $arch;
}

method _build_channel   {
    my $channel;
    try { $channel = $self->os_release_get('TAILS_CHANNEL') };
    defined $channel ? $channel : 'stable';
}

method _build_upgrade_description_file_url {
    sprintf(
        "%s/upgrade/v%d/%s/%s/%s/%s/upgrades.yml",
        $self->baseurl,
        $self->upgrade_description_url_schema_version,
        $self->product_name,
        $self->product_version,
        $self->build_target,
        $self->channel,
    );
}

method _build_upgrade_description_sig_url {
    $self->upgrade_description_file_url . '.pgp';
}

sub _build_liveos_mountpoint {
    my $self = shift;
    dir('/lib/live/mount/medium');
}

sub _build_boot_device {
    my $self = shift;

    my $device;
    try {
        $device = $self->underlying_physical_device($self->liveos_mountpoint);
    } catch {
        $self->display_error(
            $self->main_window,
            $self->encoding->decode(gettext(
                q{The device Tails is running from cannot be found. Maybe you used the `toram' option?},
            )),
            '',
        );
    };

    return $device;
}

sub _build_system_partition {
    my $self = shift;

    $self->udisks->device_partition_with_label(
        $self->boot_device,
        $self->system_partition_label
    );
}

sub _build_boot_device_vendor {
    my $self = shift;

    $self->get_device_property($self->boot_device, 'DriveVendor');
}

sub _build_boot_device_model {
    my $self = shift;

    $self->get_device_property($self->boot_device, 'DriveModel');
}


=head1 METHODS

=cut

=head2 os_release_get

Retrieve a value from os-release file,
as specified by
http://www.freedesktop.org/software/systemd/man/os-release.html

Throws an exception if not found.

=cut
method os_release_get ($key) {
    assert(-e $self->os_release_file);
    assert_like($key, qr{[_A-Z]+});

    my $fh = file($self->os_release_file)->openr;

    while (<$fh>) {
        chomp;
        if (my ($value) = (m{\A $key [=] ["] (.*) ["] \z}xms)) {
            return $value;
        }
    }

    croak(sprintf(
        "Could not retrieve value of '%s' in '%s'",
        $key, $self->os_release_file,
    ));
}

method started_from_device_installed_with_tails_installer () {
    return $self->override_started_from_device_installed_with_tails_installer
        if $self->has_override_started_from_device_installed_with_tails_installer;
    $self->device_installed_with_tails_installer($self->boot_device);
}

method started_from_writable_device () {
    assert(-d $self->dev_dir);

    -e file($self->dev_dir, 'bilibop');
}

=head2 free_memory

Returns MemFree + Buffers + Cached, in kB.

=cut
method free_memory () {
    assert(-d $self->proc_dir);
    assert(-e file($self->proc_dir, 'meminfo'));

    Sys::Statistics::Linux::MemStats->new(
        # Not supported (and ignored) by Squeeze's libsys-statistics-linux-perl.
        # This should not be a problem at runtime, but the test suite won't pass
        # on pure Squeeze. It should run fine with the backport we have uploaded
        # to squeeze-backports, though.
        files => { path => $self->proc_dir }
    )->get->{realfree};
}

no Moose;
1;

==> tails-perl5lib_0.7.6.orig.tar.gz: Tails-perl5lib-0.7.6/lib/Tails/RunningSystem.pm <==
=head1 NAME

Tails::RunningSystem - class that represents the running Tails system

=cut

package Tails::RunningSystem;

use Moose;
use MooseX::Method::Signatures;
use MooseX::Types::Moose qw( :all );
use MooseX::Types::Path::Class;
use MooseX::Has::Sugar::Saccharin;

with 'Tails::Role::HasEncoding';
with 'Tails::Role::DisplayError::Gtk2';

use 5.10.1;
use namespace::autoclean;
use warnings FATAL => 'all';
use autodie qw(:all);

use Carp;
use Carp::Assert::More;
use Data::Dumper;
use Path::Class;
use Sys::Statistics::Linux::MemStats;
use Tails::UDisks;
use Try::Tiny;

use Locale::gettext;
use POSIX;
setlocale(LC_MESSAGES, "");
textdomain("tails-perl5lib");


=head1 ATTRIBUTES

=cut

has 'upgrade_description_url_schema_version' => lazy_build ro Int;

has "$_" => lazy_build ro Str
    for (
        qw{baseurl product_name product_version build_target channel},
        qw{upgrade_description_file_url upgrade_description_sig_url},
        qw{dev_dir os_release_file proc_dir run_dir},
    );

has 'udisks' =>
    lazy_build ro 'Tails::UDisks',
    handles => [ qw{device_installed_with_tails_installer get_device_property underlying_physical_device} ];

has 'liveos_mountpoint' => (
    isa        => 'Path::Class::Dir',
    is         => 'rw',
    lazy_build => 1,
    coerce     => 1,
    documentation => q{Mountpoint of the Tails system image.},
);

has 'boot_device' =>
    lazy_build rw Str,
    documentation => q{The UDI of the physical block device where Tails is installed, e.g. /org/freedesktop/UDisks/devices/sdb.};

has 'boot_device_file' =>
    lazy_build rw Str,
    documentation => q{The path of the physical block device where Tails is installed, e.g. /dev/sdb.};

has 'system_partition' =>
    lazy_build rw Str,
    documentation => q{The UDI of the partition where Tails is installed, e.g. /org/freedesktop/UDisks/devices/sdb1.};

has 'constants' =>
    lazy_build ro 'Tails::Constants',
    metaclass => 'NoGetopt',
    handles   => [ qw{system_partition_label}];

has 'main_window' => ro 'Gtk2::Window';

foreach (qw{boot_device_vendor boot_device_model}) {
    has $_ => lazy_build ro Str;
}

has 'override_started_from_device_installed_with_tails_installer' =>
    lazy_build ro Str;


=head1 CONSTRUCTORS AND BUILDERS

=cut

method _build_upgrade_description_url_schema_version { 1 }
method _build_dev_dir         { '/dev' }
method _build_os_release_file { '/etc/os-release' }
method _build_proc_dir        { '/proc' }
method _build_run_dir         { '/var/run' }
method _build_product_name    { $self->os_release_get('TAILS_PRODUCT_NAME') }
method _build_product_version { $self->os_release_get('TAILS_VERSION_ID')   }
method _build_baseurl         { 'https://tails.boum.org' }
method _build_udisks          { Tails::UDisks->new(); }

method _build_build_target {
    my $arch = `dpkg --print-architecture`; chomp $arch; return $arch;
}

method _build_channel {
    my $channel;
    try { $channel = $self->os_release_get('TAILS_CHANNEL') };
    defined $channel ? $channel : 'stable';
}

method _build_upgrade_description_file_url {
    sprintf(
        "%s/upgrade/v%d/%s/%s/%s/%s/upgrades.yml",
        $self->baseurl,
        $self->upgrade_description_url_schema_version,
        $self->product_name,
        $self->product_version,
        $self->build_target,
        $self->channel,
    );
}

method _build_upgrade_description_sig_url {
    $self->upgrade_description_file_url . '.pgp';
}

sub _build_liveos_mountpoint {
    my $self = shift;
    dir('/lib/live/mount/medium');
}

sub _build_boot_device {
    my $self = shift;

    my $device;
    try {
        $device = $self->underlying_physical_device($self->liveos_mountpoint);
    } catch {
        $self->display_error(
            $self->main_window,
            $self->encoding->decode(gettext(
                q{The device Tails is running from cannot be found. Maybe you used the `toram' option?},
            )),
            '',
        );
    };

    return $device;
}

sub _build_boot_device_file {
    my $self = shift;

    $self->get_device_property($self->boot_device, 'DeviceFilePresentation');
}

sub _build_system_partition {
    my $self = shift;

    $self->udisks->device_partition_with_label(
        $self->boot_device,
        $self->system_partition_label
    );
}

sub _build_boot_device_vendor {
    my $self = shift;

    $self->get_device_property($self->boot_device, 'DriveVendor');
}

sub _build_boot_device_model {
    my $self = shift;

    $self->get_device_property($self->boot_device, 'DriveModel');
}


=head1 METHODS

=cut

=head2 os_release_get

Retrieve a value from os-release file,
as specified by
http://www.freedesktop.org/software/systemd/man/os-release.html

Throws an exception if not found.

=cut
method os_release_get ($key) {
    assert(-e $self->os_release_file);
    assert_like($key, qr{[_A-Z]+});

    my $fh = file($self->os_release_file)->openr;

    while (<$fh>) {
        chomp;
        if (my ($value) = (m{\A $key [=] ["] (.*) ["] \z}xms)) {
            return $value;
        }
    }

    croak(sprintf(
        "Could not retrieve value of '%s' in '%s'",
        $key, $self->os_release_file,
    ));
}

method started_from_device_installed_with_tails_installer () {
    return $self->override_started_from_device_installed_with_tails_installer
        if $self->has_override_started_from_device_installed_with_tails_installer;
    $self->device_installed_with_tails_installer($self->boot_device);
}

method started_from_writable_device () {
    assert(-d $self->dev_dir);

    -e file($self->dev_dir, 'bilibop');
}

=head2 free_memory

Returns MemFree + Buffers + Cached, in bytes.

=cut
method free_memory () {
    assert(-d $self->proc_dir);
    assert(-e file($self->proc_dir, 'meminfo'));

    Sys::Statistics::Linux::MemStats->new(
        # Not supported (and ignored) by Squeeze's libsys-statistics-linux-perl.
        # This should not be a problem at runtime, but the test suite won't pass
        # on pure Squeeze. It should run fine with the backport we have uploaded
        # to squeeze-backports, though.
        files => { path => $self->proc_dir }
    )->get->{realfree} * 1024;
}

no Moose;
1;
